# MSWord Document Product for Zope - MSDocument.py
# Copyright (C) 2001 ric BARROCA <eric@barroca.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# MSWordDocument.py: ZClass supporting MS-Word (6/95/97/2000) documents,
#					 translating them into HTML for preview (using MSWord 
#					 module) and indexing (using ZCatalog for example).
# Object Variables:
#	 self.data (string): contains the original MSWord document
#	- self.text (string): contains the generated HTML code
#	- self.name (string): contains the original document's name

from OFS import SimpleItem
from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from OFS.PropertyManager import PropertyManager
from webdav.common import rfc1123_date

import Acquisition, re, string, os

from RTF import RTFDoc

##Useful method to set up the id
def cookId(id, title, file):
    "Cook id and title"
    if not id and hasattr(file,'filename'):
        filename = file.filename
        title = title or filename
        id = filename[max(string.rfind(filename, '/'),
                          string.rfind(filename, '\\'),
                          string.rfind(filename, ':'),
                          )+1:]
    return id, title


class RTFDocument(SimpleItem.Item, Persistent, Acquisition.Implicit, PropertyManager):
    "RTF Document"
    
    meta_type = "RTF Document"
    
    manage_options = (
        (
        {'label': 'Edit',		'action': 'manage_main'},
        {'label': 'Upload',		'action': 'manage_uploadForm'},
        )
        + PropertyManager.manage_options +
        (
        {'label': 'View',		'action': ''},
        {'label': 'Security',	'action': ''},
        )
        ) 

    # Let's define some permissions
    __ac_permissions__=(
        ('View management screens', ('manage_tabs','manage_main')),
        ('Change RTF Document', ('manage_edit','manage_editForm', 'manage', 'manage_main', 'manage_uploadForm',
                                    'manage_edit', 'manage_upload', 'PUT', 'manage_historyCopy',
                                    'manage_beforeHistoryCopy', 'manage_afterHistoryCopy')),
        ('View', ('index_html', 'viewHTML', 'download', 'showImg')),
        )

    # Two useful properties (do you see others ?)
    _properties=(
        {'id':'title', 'type': 'string'},
        {'id':'name', 'type':'string'},
        ) 

    def __init__(self, id, file, title=''):
        "Initialiaze a new instance"
        self.__name__ = id
        self.id = id
        self.title = title
        if type(file) is not type(''):
            data = file.read()
        else:
            data = file
        self.name = id
        
        self.update_data(data, id)
        
    manage_addForm = HTMLFile('addForm', globals())

    def __bobo_traverse__(self, REQUEST, name):
        """ Does the picture stuff... """
        try: return getattr(self, name)
        except AttributeError: pass
        if self.imgs.has_key(name):
            self.imgname = name
            return self.showImg
        return self
    #REQUEST.RESPONSE.notFoundError(name)
    
    totoHTML = HTMLFile('toto', globals())

    def manage_add(self, id, file, title='', REQUEST=None):
        "Add a RTF Document to a folder."
        id, title = cookId(id, title, file)
        self._setObject(id, RTFDocument(id, file, title))
        #self._getOb(id).manage_upload(file)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')
            
    viewHTML = HTMLFile('index', globals())

    manage_main = HTMLFile('editForm', globals())

    def manage_edit(self, title, REQUEST=None):
        "Edit"
        self.title = title
        return MessageDialog(
            title = "Edited",
            message = "Properties for %s changed" % self.id,
            action = "manage_main",
            )

    manage_uploadForm = HTMLFile('uploadForm', globals())

    def manage_upload(self, file, REQUEST=None):
        "Upload the RTF Document and process it (update_data)"
        if type(file) is not type(''):
            data = file.read()
            name = file.filename
        else:
            data = file
            name = self.name

        self.update_data(data, name)
                
        return MessageDialog(
            title = "Uploaded",
            message = "RTF Document data for %s uploaded" % self.id,
            action = "manage_main",
            )
            
    def update_data(self, data, name):
        "Process the RTF Document"
        self.data = data
        self.size = len(data)
        self.name = name
        
        #Let's convert the word document into html
        doc = RTFDoc(self.name, self.data)
        doc.Convert()
        html = doc.getDocHTML()
        html, imgs = doc.getImgs()
        self.text = html 
        self.imgs = imgs
        if hasattr(doc, 'cleandir'): doc.cleandir()

    def download(self, REQUEST=None):
        "Send the RTF Document"
        data = self.data
        REQUEST['RESPONSE'].setHeader('Last-Modified', rfc1123_date(self._p_mtime)) 
        REQUEST['RESPONSE'].setHeader('Content-Type', 'application/rtf')
        REQUEST['RESPONSE'].setHeader('Content-Length', len(data))
        REQUEST['RESPONSE'].write(data)
        return ''
            
    def showImg(self, RESPONSE=None):
        "Show an image of the document"
        result = re.match("^.+\.(?P<ext>.+)$", self.imgname)
        ext = result.group('ext')
        if ext == 'jpg': ext = 'jpeg'
        data = self.imgs[self.imgname]
        RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime)) 
        RESPONSE.setHeader('Content-Type', 'image/%s' % ext)
        RESPONSE.setHeader('Content-Length', len(data))
        if type(data) is type(''): return data 
        while data is not None:
            RESPONSE.write(data.data)
            data=data.next
            return ''
        
    dav = re.compile('.*dav.*', re.DOTALL + re.IGNORECASE)
    def index_html(self, REQUEST=None, RESPONSE=None):
        "Send the RTF Document"
        if self.dav.match(REQUEST['HTTP_USER_AGENT']):
            self.download(REQUEST)
        else:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/viewHTML')
            return ''
        
######
## Protocols handlers
################

    #Send the MSWord document via FTP	
    manage_FTPget = download

    #	Useful method :-) for DAV and FTP clients
    def get_size(self):
        return len(self.data)
