
/*=============================================================================
   rtf2htm, a command-line program to convert RTF-formatted documents to HTML.
   Copyright (C) 2000,2001 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at tuorfa@yahoo.com.
=============================================================================*/


/*----------------------------------------------------------------------
 * Module name:    hash
 * Author name:    Zach Smith
 * Create date:    1 Sept 2000
 * Last change:    8 Oct 2000
 * Purpose:        Hash management. Words are put into a hash and an
 *                 identifier is returned. This is used to save us from
 *                 doing multiple mallocs for recurring strings such as
 *                 'the' and \par. This is not a big issue under Unix,
 *                 but as they say, waste not want not.
 *----------------------------------------------------------------------
 * Changes:
 * tuorfa@yahoo.com, 8 Apr 01, check for out of memory after malloc.
 * tuorfa@yahoo.com, 21 Apr 01, signed to conversion unsigned bug
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "error.h"


typedef struct _hi {
	struct _hi *next;
	char *str;
	unsigned long value;
}
HashItem;


/* Index by first char of string */
static HashItem *hash[256];
static unsigned long hash_length[256];
static unsigned long hash_value=0;


void
hash_init ()
{
	int i;
	for (i=0; i<256; i++) {
		hash[i]=NULL;
		hash_length[i]=0;
	}
}


void
hash_stats ()
{
	int i;
	unsigned long total=0;
	for (i=0; i<256; i++) {
		total += hash_length[i];
	}
	fprintf (stderr,"%lu words were hashed.\n", total);
}


static HashItem *
hashitem_new (char *str)
{
	HashItem *hi;
	hi=(HashItem*)malloc(sizeof(HashItem));
	if (!hi)
		error_handler ("out of memory");
	bzero ((void*)hi, sizeof (HashItem));
	hi->str = strdup(str);
	hi->value = (str[0]<<24) | (hash_value++ & 0xffffff);
	hi->next = NULL;
	return hi;
}

unsigned long
hash_get_index (char *str)
{
	int index;
	HashItem *hi;
	index = (unsigned char) str[0];
	hi = hash[index];
	while (hi) {
		if (!strcmp(hi->str,str))
			return hi->value;
		hi=hi->next;
	}
	/* not in hash */
	hi = hashitem_new (str);
	hi->next = hash[index];
	hash [index] = hi;
	++hash_length [index];
	return hi->value;
}

char*
hash_get_string (unsigned long value)
{
	int index;
	HashItem *hi;
	index = value >> 24;
	hi = hash[index];
	while (hi) {
		if (hi->value == value)
			return hi->str;
		hi=hi->next;
	}
	return NULL;
}



