
/*=============================================================================
   rtf2htm, a command-line program to convert RTF-formatted documents to HTML.
   Copyright (C) 2000,2001 Zachary Thayer Smith

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   The author is reachable by electronic mail at tuorfa@yahoo.com.
=============================================================================*/


/*----------------------------------------------------------------------
 * Module name:    main.c
 * Author name:    Zach Smith
 * Create date:    1 Sept 2000
 * Last change:    15 Oct 2000
 * Purpose:        main() routine with file open/close.
 *----------------------------------------------------------------------
 * Changes:
 * tuorfa@yahoo.com, 14 Oct 2000: added -nopict option
 * tuorfa@yahoo.com, 15 Oct 2000: added verify_file_type() 
 * tuorfa@yahoo.com, 8 Apr 2001: more GNU-like switches implemented
 *--------------------------------------------------------------------*/



#include <stdio.h>
#include <string.h>

#include "defs.h"
#include "error.h"
#include "word.h"
#include "parse.h"
#include "hash.h"


int nopict_mode; /* TRUE => do not write \pict's to files */
int dump_mode;   /* TRUE => output a dump of the word tree */
int debug_mode;  /* TRUE => output comments within HTML */
int lineno;      /* used for error reporting */
int simple_mode;  /* TRUE => output HTML without SPAN/DIV tags */


/* After accidentally running rtf2htm on HTML files for the 20th time,
 * I decided to add this function.
 */
int
verify_file_type (char *path) {
	char *s;
	char *s2=NULL;
	int ch;
	FILE *f;
	CHECK_PARAM_NOT_NULL(path);

	/* Get the file extension */
	s=path;
	while((s = strchr(s,'.'))) {
		s2=s;
		s++;
	}

	/* Not rtf? wrong file type */
	if (s2) {
		if (strcmp(s2,".rtf") && strcmp (s2,".RTF"))
			return FALSE;
	}
	
	/* If RTF or no file extension, look for "{" only (very simple test
	 * but few file types use a brace as the first char, and the spec
	 * I have does not say \rtf1 must be the first symbol).
	 */
	f=fopen(path,"r");
	if (!f) return FALSE;
	while(EOF!=(ch=fgetc(f))) {
		if (ch>' ') break;
	}
	fclose(f);

	return ch=='{' ? TRUE : FALSE;
}


int
main (int argc, char **argv)
{
	FILE *f;
	Word * word;
	char *path=NULL;
	int i;

	nopict_mode = debug_mode = dump_mode = FALSE;

	if (argc<2 || argc>7) usage();

	for (i=1; i<argc; i++) {
		if (!strcmp("--dump",argv[i])) dump_mode=TRUE;
		else if (!strcmp("-d",argv[i])) dump_mode=TRUE;
		else if (!strcmp("--debug",argv[i])) debug_mode=TRUE;
		else if (!strcmp("--simple",argv[i])) simple_mode=TRUE;
		else if (!strcmp("--help",argv[i]))  {
			usage();
			exit (0);
		}
		else if (!strcmp("--version",argv[i]))  {
			fprintf (stderr, "This is rtf2htm version %s\n", PROGRAM_VERSION);
			exit (0);
		}
		else if (!strcmp("--nopict",argv[i])) nopict_mode=TRUE;
		else if (!strcmp("-n",argv[i])) nopict_mode=TRUE;
		else {
			if (*argv[i]=='-') usage();

			if(path) 
				usage();
			else 	
				path=argv[i];
		}
	}
	
	if (!path) usage();

	if (!verify_file_type (path)) 
		error_handler ("input file must be RTF\n");

	hash_init();

	fprintf (stderr, "This is rtf2htm, ");
	fprintf (stderr, "version %s\n", PROGRAM_VERSION);
	fprintf (stderr, "By Zach T. Smith\n");

	f = fopen (path, "r");
	if (!f) {
		char path2[200];
		strcpy(path2,path); strcat(path2,".rtf");
		f = fopen(path2, "r");
		if(!f)
			error_handler ("cannot open input file");
	}

	fprintf(stderr,"Processing %s...\n", path);

	lineno=0;

	word = word_read (f);

	if (dump_mode) {
		word_dump (word);
	} else {
		word_print_html (word);
	}

	fclose(f);

	fprintf(stderr,"Done.\n");

	hash_stats();

	return 0;
}


